package com.ruoyi.service.impl;

import java.util.List;

import cn.hutool.core.util.ObjUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.framework.web.domain.server.Sys;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.web.request.AutomobileEnterpriseListRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.mapper.AutomobileEnterpriseMapper;
import com.ruoyi.domain.AutomobileEnterprise;
import com.ruoyi.service.AutomobileEnterpriseService;
import org.springframework.transaction.annotation.Transactional;

/**
 * 车企信息Service业务层处理
 *
 * @author gxk
 * @date 2024-01-05
 */
@Service
@Transactional
public class AutomobileEnterpriseServiceImpl extends ServiceImpl<AutomobileEnterpriseMapper, AutomobileEnterprise> implements AutomobileEnterpriseService
{
    @Autowired
    private AutomobileEnterpriseMapper automobileEnterpriseMapper;

    @Autowired
    private ISysDeptService deptService;

    /**
     * 查询车企信息
     *
     * @param id 车企信息主键
     * @return 车企信息
     */
    @Override
    public AutomobileEnterprise selectAutomobileEnterpriseById(Long id)
    {
        return automobileEnterpriseMapper.selectAutomobileEnterpriseById(id);
    }

    /**
     * 查询车企信息列表
     *
     * @param request 车企信息
     * @return 车企信息
     */
    @Override
    public List<AutomobileEnterprise> selectAutomobileEnterpriseList(AutomobileEnterpriseListRequest request)
    {
        return automobileEnterpriseMapper.selectAutomobileEnterpriseList(request);
    }

    /**
     * 新增车企信息
     *
     * @param automobileEnterprise 车企信息
     * @return 结果
     */
    @Override
    public int insertAutomobileEnterprise(AutomobileEnterprise automobileEnterprise)
    {
        // 判断企业名称是否重复
        if (lambdaQuery()
                .eq(AutomobileEnterprise::getEnterpriseName, automobileEnterprise.getEnterpriseName())
                .ne(ObjUtil.isNotEmpty(automobileEnterprise.getId()), AutomobileEnterprise::getId, automobileEnterprise.getId())
                .exists()
        ) {
            throw new ServiceException("企业名称已存在");
        }
        automobileEnterprise.setCreateTime(DateUtils.getNowDate());
        automobileEnterprise.setCreateBy(String.valueOf(SecurityUtils.getUserId()));
        automobileEnterpriseMapper.insert(automobileEnterprise);
        SysDept dept = new SysDept();
        dept.setOrderNum(0);
        dept.setParentId(100l);
        dept.setAncestors("0,100");
        dept.setDeptType("enterprise");
        dept.setDeptName(automobileEnterprise.getEnterpriseName());
        dept.setAddress(automobileEnterprise.getAddress());
        dept.setPostcode(automobileEnterprise.getPostcode());
        dept.setLeader(automobileEnterprise.getEnterpriseContact());
        dept.setPhone(automobileEnterprise.getContactNumber());
        dept.setAutomobileEnterpriseId(automobileEnterprise.getId());
        deptService.insertDept(dept);
        return 1;
    }

    /**
     * 修改车企信息
     *
     * @param automobileEnterprise 车企信息
     * @return 结果
     */
    @Override
    public int updateAutomobileEnterprise(AutomobileEnterprise automobileEnterprise)
    {
        // 判断企业名称是否重复
        if (lambdaQuery()
                .eq(AutomobileEnterprise::getEnterpriseName, automobileEnterprise.getEnterpriseName())
                .ne(ObjUtil.isNotEmpty(automobileEnterprise.getId()), AutomobileEnterprise::getId, automobileEnterprise.getId())
                .exists()
        ) {
            throw new ServiceException("企业名称已存在");
        }
        automobileEnterprise.setUpdateTime(DateUtils.getNowDate());
        automobileEnterprise.setUpdateBy(String.valueOf(SecurityUtils.getUserId()));
        automobileEnterpriseMapper.updateAutomobileEnterprise(automobileEnterprise);
        SysDept dept = deptService.getDeptByAutomobileEnterpriseId(automobileEnterprise.getId());
        dept.setDeptName(automobileEnterprise.getEnterpriseName());
        dept.setAddress(automobileEnterprise.getAddress());
        dept.setPostcode(automobileEnterprise.getPostcode());
        dept.setLeader(automobileEnterprise.getEnterpriseContact());
        dept.setPhone(automobileEnterprise.getContactNumber());
        deptService.updateDept(dept);
        return 1;
    }

}
