package com.ruoyi.web;


import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelMultipleSheetsUtil;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.domain.ReviewStandard;
import com.ruoyi.domain.TTestPlanRecord;
import com.ruoyi.domain.vo.ItemsByUseCaseVO;
import com.ruoyi.domain.vo.RecaordUseCasesReportVO;
import com.ruoyi.domain.vo.RecaordUseCasesVO;
import com.ruoyi.mapper.TTestPlanRecordMapper;
import com.ruoyi.service.TTestPlanRecordService;
import com.ruoyi.service.impl.ReviewStandardServiceImpl;
import com.ruoyi.web.request.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.skywalking.apm.toolkit.trace.Tag;
import org.apache.skywalking.apm.toolkit.trace.Tags;
import org.apache.skywalking.apm.toolkit.trace.Trace;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import com.ruoyi.common.core.controller.BaseController;
import org.springframework.web.bind.annotation.RequestBody;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.*;


@Api(tags = "检测方案记录")
@RestController
@RequestMapping("/Plan/Record")
public class TestPlanRecordController extends BaseController{
    private static final String XLSX_SUFFIX = ".xlsx";
    @Autowired
    private TTestPlanRecordService tTestPlanRecordService;

    @Autowired
    private ReviewStandardServiceImpl reviewStandardService;

    @Autowired
    private TTestPlanRecordMapper tTestPlanRecordMapper;

    @ApiOperation("新增检测方案")
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @Log(title = "新增检测方案", businessType = BusinessType.INSERT)
    @RequestMapping(method = RequestMethod.POST, value = "/add")
    public R<String> insertTestPlanRecord(@Validated @RequestBody TTestPlanRecord tTestPlanRecord){
        tTestPlanRecordService.insertTestPlanRecord(tTestPlanRecord);
        return R.ok(tTestPlanRecord.getId().toString());
    }

    @ApiOperation("检测方案详情")
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @RequestMapping(method = RequestMethod.POST, value = "/details")
    public R<TTestPlanRecord> selectTestPlanRecordDetails(@Validated @RequestBody TestPlanRecordGetInfoRequest testPlanRecordGetInfoRequest){
        return R.ok(tTestPlanRecordService.selectTestPlanRecordDetails(testPlanRecordGetInfoRequest.getId()));
    }


    @ApiOperation("查询标准详情")
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @RequestMapping(method = RequestMethod.POST, value = "/standard")
    public R<List<ReviewStandard>> selectStandarDetails(@Validated @RequestBody PlanDetailStandardRequest planDetailStandardRequest){
        List<ReviewStandard> tree = reviewStandardService.findTree(planDetailStandardRequest.getInspectionStandardId(), planDetailStandardRequest.getType());
        return R.ok(tree);
    }

    @ApiOperation("查询检验记录")
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @RequestMapping(method = RequestMethod.POST, value = "/page")
    public TableDataInfo<TTestPlanRecord> selectTestPlanRecordPage(@Validated @RequestBody TestPlanRecordPageRequest testPlanRecordPageRequest){
        startPage(testPlanRecordPageRequest);
        List<TTestPlanRecord> list = tTestPlanRecordService.selectTestPlanRecordPage(testPlanRecordPageRequest);
        return getDataTable(list);
    }

    @ApiOperation("更新检验方案")
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @RequestMapping(method = RequestMethod.POST, value = "/update")
    public R<Integer> updateTestPlanRecord(@Validated @RequestBody  TestPlanRecordGetInfoRequest testPlanRecordGetInfoRequest){
        return R.ok(tTestPlanRecordService.updateTestPlanRecord(testPlanRecordGetInfoRequest));
    }


    @ApiOperation("车型实验详情")
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @RequestMapping(method = RequestMethod.POST, value = "/useCase")
    public R<List> getUseCases(@Validated @RequestBody String request ) throws IOException {
        return R.ok(tTestPlanRecordService.getCarTestDetail(request));
    }

    @ApiOperation("检验记录删除")
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @RequestMapping(method = RequestMethod.POST, value = "/del")
    public R<Integer> getUseCases(@Validated @RequestBody TestPlanRecordGetInfoRequest testPlanRecordGetInfoRequest ) {
        return R.ok(tTestPlanRecordMapper.deleteById(testPlanRecordGetInfoRequest.getId()));
    }

    @ApiOperation("导出检验记录")
    @Trace
    @Tags({@Tag(key = "param", value = "arg[0]"), @Tag(key = "result", value = "returnedObj")})
    @RequestMapping(method = RequestMethod.POST, value = "/export")
    public void exportPlanRecord(HttpServletResponse respon,@Validated @RequestBody String params) throws IOException{

        List<ItemsByUseCaseVO> listUseCase  =  tTestPlanRecordService.getCarTestDetail(params);
        List<RecaordUseCasesVO>  list = new ArrayList<>();
        listUseCase.forEach(item->{
            item.getUseCases().forEach(data->{
                list.add(data);
            });
        });
        ExcelUtil<RecaordUseCasesVO> util = new ExcelUtil<RecaordUseCasesVO>(RecaordUseCasesVO.class);
        util.exportExcel( respon, list, "用例编号");
//        return R.ok();
    }

    @RequestMapping(method = RequestMethod.POST, value = "/exportTo")
    public void exportPlanRecordTo(HttpServletResponse respon, @Validated @RequestBody SystemReviewExportRequest systemReviewExportRequest) throws IOException{

      List inspectionTtem = systemReviewExportRequest.getInspectionTtem();
        Map<String, Object> result = new HashMap<>();
        Boolean isPage = true;
        inspectionTtem.forEach(item->{
            if("sraif".equals(item)){
                result.put(isPage ? "体系审查" : "sheet1名字", tTestPlanRecordMapper.systemReviewExport(systemReviewExportRequest.getInspectionStandardId(), "system") );
            }
            if ("grfis".equals(item)){
                result.put(isPage ? "车型审查" : "sheet1名字", tTestPlanRecordMapper.systemReviewExport(systemReviewExportRequest.getInspectionStandardId(), "car") );
            }
            if ("trfis".equals(item)){
                String params = JSON.toJSONString(systemReviewExportRequest.getCarExperimentRequest());
                List<ItemsByUseCaseVO> listUseCase  = new ArrayList<>();
                try {
                    listUseCase = tTestPlanRecordService.getCarTestDetail(params);
                } catch (IOException e) {
                    throw new RuntimeException(e);
                }
                List<RecaordUseCasesReportVO>  list = new ArrayList<>();
                listUseCase.forEach(itemS->{
                    itemS.getUseCases().forEach(data->{
                        RecaordUseCasesReportVO recaordUseCasesReportVO = new RecaordUseCasesReportVO();
                        recaordUseCasesReportVO.setSerialNumber(itemS.getSerialNumber());
                        recaordUseCasesReportVO.setRequirements(itemS.getRequirements());
                        recaordUseCasesReportVO.setCustomizedID(data.getCustomizedID());
                        recaordUseCasesReportVO.setDescription(data.getDescription());
                        list.add(recaordUseCasesReportVO);
                    });
                });
                result.put(isPage ? "车型检验" : "sheet1名字", list );
            }
        });
        Map<String, String> cover = new HashMap<>();
        cover.put("检测标准:",systemReviewExportRequest.getInspectionStandardName());
        cover.put("被检车企:",systemReviewExportRequest.getInspectCarCompanyName());
        cover.put("车辆VIN号:",systemReviewExportRequest.getCarModel());
        cover.put("检测机构:",systemReviewExportRequest.getTestOrganizationName());
        cover.put("生成时间:",systemReviewExportRequest.getCreateTime());

        // 自定义排序顺序
        List<String> customOrder = Arrays.asList("体系审查", "车型审查", "车型检验");
        Map<String, Object> resultDesc = new LinkedHashMap<>();
        customOrder.forEach(key -> {
            if (result.containsKey(key)) {
                resultDesc.put(key, result.get(key));
            }else {
                resultDesc.put(key, new ArrayList<>());
            }
        });
        ExcelMultipleSheetsUtil.excelMultipleSheets(resultDesc, "检测方案" + DateUtil.format(new Date(), "yyyyMMdd"), XLSX_SUFFIX, respon,cover);

    }

}
